@Initialization of all Gaussian random number generators by the global variable dseed@
dseed = 10^8;
@ ==============================================================================================
Procedure 	: 	h
Authors		: 	Christian M. Dahl and Gloria Gonzalez-Rivera
Last edited	: 	April 27., 2001

Input		:	x a (kx1) vector 
Output		:	Half the Euclidian distance between the (kx1) elements in x
=============================================================================================== @
proc(1) = h(x); 
  retp(0.5*sqrt(sumc(x^2)));
endp; 


@ ===============================================================================================
Procedure 	: 	covsph...slow!!!used for computation of tests only
Authors		: 	Christian M. Dahl and Gloria Gonzalez-Rivera
Last edited	: 	April 27., 2001

Input		:	x a (Txk) vector 
Input		:	&h (The procedure computing half the Euclidian distance between the (kx1) 
			elements in x)
Output		:	Hamilton's spherical covariance function; a (TxT) matrix

Notice		:	Only works for k<6
=============================================================================================== @
proc(1) = covsph(x,&h);
  local h:proc;
  local nx,capT,tmp,g,t,s,i,j;
  
  nx   			= cols(x);
  capT 			= rows(x);
  	
  tmp  			= zeros(capT,capT);
  g    			= 2*(sqrt(nx)*stdc(x))^(-1);
  g    			= g';
  
  i 			= 1;
  do until i > capT;
    j 			= i;
    do until j > capT;
    
      if nx==1;
        t 		= (g.*x[i,.])';
        s 		= (g.*x[j,.])';
        tmp[i,j] 	= (h(t-s)<1)*(1-h(t-s));
        tmp[j,i] 	= tmp[i,j];
      endif;
      
      if nx==2;
        t 		= (g.*x[i,.])';
        s 		= (g.*x[j,.])';
        tmp[i,j] 	= (h(t-s).<1)*(1-(2/pi)*(h(t-s)*sqrt(1-h(t-s)^2)+arcsin(h(t-s))));
        tmp[j,i] 	= tmp[i,j];
      endif;
      
      if nx==3;
        t 		= (g.*x[i,.])';
        s 		= (g.*x[j,.])';
        tmp[i,j]	= (h(t-s)<1)*(1-(1/2)*(3*h(t-s)-h(t-s)^3));
        tmp[j,i] 	= tmp[i,j];
      endif;
      
      if nx==4;
        t 		= (g.*x[i,.])';
        s 		= (g.*x[j,.])';
        if h(t-s)<1;
          tmp[i,j] 	= (1-(2/pi)*((2/3)*h(t-s)*(1-h(t-s)^2)^(1.5)+h(t-s)*sqrt(1-h(t-s)^2)+arcsin(h(t-s))));
          tmp[j,i] 	= tmp[i,j];
        endif;          
      endif;
      
      if nx==5;
        t 		= (g.*x[i,.])';
        s 		= (g.*x[j,.])';
        tmp[i,j] 	= (h(t-s)<1)*(1-(3/2)*h(t-s)+(1/2)*h(t-s)^3-(3/8)*h(t-s)*(1-h(t-s)^2)^2);
        tmp[j,i] 	= tmp[i,j];
      endif;
      j 		= j + 1;
    endo;
    i 			= i + 1;
  endo;
  
  retp(tmp);
endp; 

@ ===============================================================================================
Procedure 	: 	remidel
Authors		: 	Christian M. Dahl and Gloria Gonzalez-Rivera
Last edited	: 	April 27., 2001

Input		:	x: (Txk) vector 
Output		:	(Txs) vector with distinct columns 

Notice		:	Procedure that removes identical columns in the matrix x
=============================================================================================== @ 
proc(1) = remidel(x);
  local colsx,rowsx,indx,c,jter,indx1,cs,precis;

  precis  		= 10^(-5);
  indx1   		= 0;  
  colsx   		= cols(x);
  rowsx   		= rows(x);

  cs      		= sumc(x);

  indx    		= seqa(1,1,cols(x));
  c       		= indx~cs;
  c       		= sortc(c,2);
  c       		= c';
  
  jter    		= 1;
  indx1   		= c[1,1];
  do until jter == cols(x);
    if c[2,jter+1] > c[2,jter]+precis;
      indx1 		= indx1|c[1,jter+1];
    endif;
    jter  		= jter + 1;
  endo;  
  indx1   		= sortc(indx1,1);
  
  retp(x[.,indx1']); 
endp; 

@ ===============================================================================================
Procedure 	: 	Computes the original test by Hamilton (2001) + asymptotic p-value +
			bootstrapped p-value.
Authors		: 	Christian M. Dahl and Gloria Gonzalez-Rivera
Last edited	: 	Nov. 24, 2001 

Input			y 		: (Tx1) vector of the endogenuous variable
			x 		: (Txk) matrix of the explanatory variables
			xnon		: (rx1) vector indexing the components in x that enter
					  the nonlinear part (non-zero g's). 
					  Example: If k = 5 and x1, x2, and x5 should enter the 
					  nonlinear part (r = 3) then specify xnon = 1|2|5.
			boot		: Number of replications required to compute the bootstrapped
					  p-value associated with the test statistic.
			
Output			torg 		: The testor
			p_asymp		: Asymptotic p-value
			p_boot		: Bootstrapped p-value
=============================================================================================== @
proc(3) = hamiltontestbootc(y,x,xnon,boot);
  local capT,k0,xwhole,m0,epshat,sigeps,ht,a0,zeta,yf;  
  local torg,init,beta,rejection,iter,jter,tstar,sigepsb,epshatb;
  local betab,p_asymp,p_boot;
  
  capT   		= rows(x);
  k0     		= cols(x);
  
  tstar  		= zeros(boot,1);
  
  xwhole 		= ones(capT,1)~x;
  
  beta   		= invpd(xwhole'xwhole)*xwhole'y;
  m0     		= xwhole*invpd(xwhole'xwhole)*xwhole';
  m0     		= eye(capT)-m0;
  
  epshat 		= m0*y;
        
  sigeps 		= (epshat'epshat)/(capT-k0-1);
  ht     		= covsph(x[.,xnon],&h);
  
  a0     		= m0*ht*m0;
  zeta   		= a0-m0*sumc(diag(a0))/(capT-k0-1);
  zeta   		= sqrt(2*sumc(diag(zeta*zeta)));
  zeta   		= (epshat'ht*epshat-sigeps*sumc(diag(a0)))/(sigeps*zeta);
  torg   		= zeta*zeta;
  
  iter 			= 1;
  rejection 		= 0;
  do until iter > boot;
    epshatb		= sqrt(sigeps)*rndn(capT,1);
    y      		= xwhole*beta+epshatb;
  
    betab   		= invpd(xwhole'xwhole)*xwhole'y;
    m0      		= xwhole*invpd(xwhole'xwhole)*xwhole';
    m0      		= eye(capT)-m0;
  
    epshat       	= m0*y;
        
    sigepsb      	= (epshat'epshat)/(capT-k0-1);
  
    a0           	= m0*ht*m0;
    zeta         	= a0-m0*sumc(diag(a0))/(capT-k0-1);
    zeta         	= sqrt(2*sumc(diag(zeta*zeta)));
    zeta         	= (epshat'ht*epshat-sigepsb*sumc(diag(a0)))/(sigepsb*zeta);
    tstar[iter]  	= zeta*zeta;
     
       
    rejection 		= rejection + (tstar[iter]>torg);
    iter 		= iter + 1;
  endo;  
    
  p_asymp 		= cdfchic(torg,1);
  p_boot  		= ((1+rejection)/(1+boot));
  
  retp(torg,p_asymp,p_boot);
endp;

@ ===============================================================================================
Procedure 	: 	Computes the lambda(A)-test by Dahl and Gloria Gonzalez-Rivera (2000) 
			+ asymptotic p-value +	bootstrapped p-value.
Authors		: 	Christian M. Dahl and Gloria Gonzalez-Rivera
Last edited	: 	Nov. 24, 2001 

Input			y 		: (Tx1) vector of the endogenuous variable
			x 		: (Txk) matrix of the explanatory variables
			xnon		: (rx1) vector indexing the components in x that enter
					  the nonlinear part (non-zero g's). 
					  Example: If k = 5 and x1, x2, and x5 should enter the 
					  nonlinear part (r = 3) then specify xnon = 1|2|5.
			boot		: Number of replications required to compute the bootstrapped
					  p-value associated with the test statistic.
			
Output		:	torg 		: The testor
			p_asymp		: Asymptotic p-value
			p_boot		: Bootstrapped p-value
=============================================================================================== @
proc(3)=Alambdatestbootc(y,x,xnon,boot);
  local nobs,xwhole,eps,SSR0,SSR1,sigmasq,uhat,vhat,xc,gam,Rsq,I,g;
  local stat,iter,jter,Z,ttemp,indx,nx,auxwhole,yf,numb;
  local beta,xtest,torg,tstar,epshat,epshatb,betab,sigmasqb,rejection,capT;  
  local p_asymp,p_boot;
  
  capT  		= rows(x);
  xtest 		= x[.,xnon];
  tstar 		= zeros(boot,1);
        
  nobs    		= rows(y);  
  xwhole  		= ones(nobs,1)~x;
  
  beta  		= invpd(xwhole'xwhole)*(xwhole'y);
  eps   		= y-xwhole*beta;
   
  sigmasq 		= meanc(eps^2);  
  uhat    		= vec(eye(nobs)-(eps*eps')./sigmasq);
  SSR0    		= uhat'uhat;
  
  iter    		= 1;
  nx      		= cols(xtest);
  do until iter > nx;
    xc    		= xtest[.,iter].*ones(nobs,nobs);
    if iter == 1;
      Z   		= vec(abs(xc-xc'));     
    else;
      Z   		= Z~vec(abs(xc-xc'));
    endif;
    iter  		= iter + 1;
  endo;	
  
  if nx > 1;
    auxwhole 		= Z~remidel(Z*~Z);
  else;
    auxwhole 		= Z~Z^2;
  endif;	
  
  numb       		= cols(auxwhole)+1;
  auxwhole   		= ones(nobs^2,1)~auxwhole~vec(eye(nobs));
  
  vhat       		= uhat-auxwhole*invpd(auxwhole'auxwhole)*(auxwhole'uhat);
  
  SSR1       		= vhat'vhat;  
  Rsq        		= (SSR0-SSR1)/SSR0;  
  torg       		= rows(Z)*Rsq;

  iter 			= 1;
  rejection 		= 0;
  do until iter > boot;
    epshatb 		= sqrt(sigmasq)*rndn(capT,1);
    y       		= xwhole*beta+epshatb;
  
    betab   		= invpd(xwhole'xwhole)*xwhole'y;
    epshat  		= y-xwhole*betab;
    sigmasqb		= meanc(epshat^2);  
    uhat    		= vec(eye(nobs)-(epshat*epshat')./sigmasqb);    
    SSR0    		= uhat'uhat;   
    
    vhat       		= uhat-auxwhole*invpd(auxwhole'auxwhole)*(auxwhole'uhat);
      
    SSR1       		= vhat'vhat;  
    Rsq        		= (SSR0-SSR1)/SSR0;  
    tstar[iter] 	= rows(Z)*Rsq;
        
    rejection 		= rejection + (tstar[iter]>torg);
    iter 		= iter + 1;
  endo;  
    
  p_asymp 		= cdfchic(torg,numb);
  p_boot  		= ((1+rejection)/(1+boot));
  
  retp(torg,p_asymp,p_boot);
endp;

@ ===============================================================================================
 Procedure 	: 	Computes the lambda(E)-test by Dahl and Gloria Gonzalez-Rivera (2000) 
 			+ asymptotic p-value +	bootstrapped p-value.
 Authors	: 	Christian M. Dahl and Gloria Gonzalez-Rivera
 Last edited	: 	Nov. 24, 2001 
 
 Input			y 		: (Tx1) vector of the endogenuous variable
 			x 		: (Txk) matrix of the explanatory variables
 			xnon		: (rx1) vector indexing the components in x that enter
 					  the nonlinear part (non-zero g's). 
 					  Example: If k = 5 and x1, x2, and x5 should enter the 
 					  nonlinear part (r = 3) then specify xnon = 1|2|5.
 			boot		: Number of replications required to compute the bootstrapped
 					  p-value associated with the test statistic.
 			
 Output			torg 		: The testor
 			p_asymp		: Asymptotic p-value
 			p_boot		: Bootstrapped p-value
=============================================================================================== @
proc(3)=Elambdatestbootc(y,x,xnon,boot);
  local nobs,xwhole,eps,SSR0,SSR1,sigmasq,uhat,vhat,xc,gam,Rsq,I,g,stat;
  local iter,jter,Z,ttemp,indx,nx,auxwhole,yf,hplus,hoo;  
  local beta,xtest,torg,tstar,epshat,epshatb,betab,sigmasqb,rejection;  
  local p_asymp,p_boot;
  
  nobs 			= rows(y);
  
  tstar 		= zeros(boot,1);
  
  xtest 		= x[.,xnon];
  nx 			= cols(xtest);
  
  xwhole 		= ones(nobs,1)~x;  
  beta 			= invpd(xwhole'xwhole)*(xwhole'y);
  eps  			= y-xwhole*beta;  
    
  sigmasq 		= meanc(eps^2);
  
  uhat 			= vec(eye(nobs)-(eps*eps')./sigmasq);
  SSR0 			= uhat'uhat;
  
  iter 			= 1;  
  do until iter > nx;    
    xc 			= xtest[.,iter].*ones(nobs,nobs);
    if iter == 1;
      Z 		= vec(abs(xc-xc'));     
    else;
      Z 		= Z~vec(abs(xc-xc'));
    endif;
    iter 		= iter + 1;
  endo;	  
  
  gam 			= meanc(abs(xtest-meanc(xtest)'));     
  gam 			= 2./(nx*gam);
  
  hplus 		= 0.5*(Z*gam);
  
  hoo 			= (1-hplus)^rows(xnon);
  I 			= (hplus.<=1 );
  auxwhole 		= (hoo.*I)~vec(eye(nobs));  
  
  vhat 			= uhat-auxwhole*invpd(auxwhole'auxwhole)*(auxwhole'uhat);
  SSR1 			= vhat'vhat;
  Rsq 			= (SSR0-SSR1)/SSR0;
  
  torg 			= rows(Z)*Rsq;
  
  iter 			= 1;
  rejection 		= 0;
  do until iter > boot;
    epshatb 		= sqrt(sigmasq)*rndn(nobs,1);
    y       		= xwhole*beta+epshatb;
  
    betab   		= invpd(xwhole'xwhole)*xwhole'y;
    epshat  		= y-xwhole*betab;
    sigmasqb		= meanc(epshat^2);  
    uhat    		= vec(eye(nobs)-(epshat*epshat')./sigmasqb);    
    SSR0    		= uhat'uhat;   
    
    vhat       		= uhat-auxwhole*invpd(auxwhole'auxwhole)*(auxwhole'uhat);      
    SSR1       		= vhat'vhat;  
    
    Rsq        		= (SSR0-SSR1)/SSR0;  
    tstar[iter] 	= rows(Z)*Rsq;
        
    rejection 		= rejection + (tstar[iter]>torg);
    iter 		= iter + 1;
  endo;  
    
  p_asymp 		= cdfchic(torg,1);
  p_boot  		= ((1+rejection)/(1+boot));
  
  retp(torg,p_asymp,p_boot);
endp;

@ ===============================================================================================
 Procedure 	: 	Computes the g-test by Dahl and Gloria Gonzalez-Rivera (2000) 
 			+ asymptotic p-value +	bootstrapped p-value.
 Authors	: 	Christian M. Dahl and Gloria Gonzalez-Rivera
 Last edited	: 	Nov. 24, 2001 
 
 Input			y 		: (Tx1) vector of the endogenuous variable
 			x 		: (Txk) matrix of the explanatory variables
 			xnon		: (rx1) vector indexing the components in x that enter
 					  the nonlinear part (non-zero g's). 
 					  Example: If k = 5 and x1, x2, and x5 should enter the 
 					  nonlinear part (r = 3) then specify xnon = 1|2|5.
 			boot		: Number of replications required to compute the bootstrapped
 					  p-value associated with the test statistic.
 			
 Output			torg 		: The testor
 			p_asymp		: Asymptotic p-value
 			p_boot		: Bootstrapped p-value
=============================================================================================== @
proc(3)=gtestbootc(y,x,xnon,boot);
  local nobs,xwhole,eps,SSR0,SSR1,sigmasq,uhat,vhat,xc,gam,Rsq,I,g;
  local stat,iter,jter,Z,ttemp,indx,nx,auxwhole,yf,numb;
  local beta,xtest,torg,tstar,epshat,epshatb,betab,sigmasqb,rejection,capT;  
  local p_asymp,p_boot;
  
  xtest 		= x[.,xnon];
  tstar 		= zeros(boot,1);
        
  nobs    		= rows(y);  
  xwhole  		= ones(nobs,1)~x;
  
  beta  		= invpd(xwhole'xwhole)*(xwhole'y);
  eps   		= y-xwhole*beta;
   
  sigmasq 		= meanc(eps^2);  
  uhat    		= vec(eye(nobs)-(eps*eps')./sigmasq);
  SSR0    		= uhat'uhat;
  
  iter    		= 1;
  nx      		= cols(xtest);
  do until iter > nx;
    xc    		= xtest[.,iter].*ones(nobs,nobs);
    if iter == 1;
      Z   		= vec(abs(xc-xc'))-vec(abs(xc))-vec(abs(xc'));
    else;
      Z   		= Z~(vec(abs(xc-xc'))-vec(abs(xc))-vec(abs(xc')));
    endif;
    iter  		= iter + 1;
  endo;	
  
  if nx > 1;
    auxwhole 		= Z~remidel(Z*~Z);
  else;
    auxwhole 		= Z~Z^2;
  endif;	
  
  numb       		= cols(auxwhole);
  auxwhole   		= auxwhole~vec(eye(nobs));
  
  vhat       		= uhat-auxwhole*invpd(auxwhole'auxwhole)*(auxwhole'uhat);
  
  SSR1       		= vhat'vhat;  
  Rsq        		= (SSR0-SSR1)/SSR0;  
  torg       		= rows(Z)*Rsq;

  iter 			= 1;
  rejection 		= 0;
  do until iter > boot;
    epshatb 		= sqrt(sigmasq)*rndn(nobs,1);
    y       		= xwhole*beta+epshatb;
  
    betab   		= invpd(xwhole'xwhole)*xwhole'y;
    epshat  		= y-xwhole*betab;
    sigmasqb		= meanc(epshat^2);  
    uhat    		= vec(eye(nobs)-(epshat*epshat')./sigmasqb);    
    SSR0    		= uhat'uhat;   
    
    vhat       		= uhat-auxwhole*invpd(auxwhole'auxwhole)*(auxwhole'uhat);
      
    SSR1       		= vhat'vhat;  
    Rsq        		= (SSR0-SSR1)/SSR0;  
    tstar[iter]		= rows(Z)*Rsq;
        
    rejection 		= rejection + (tstar[iter]>torg);
    iter 		= iter + 1;
  endo;  
    
  p_asymp 		= cdfchic(torg,numb);
  p_boot  		= ((1+rejection)/(1+boot));
  
  retp(torg,p_asymp,p_boot);
endp;
