@ This Gauss program evaluates the effects of lags 3 and 4 for various nonlinear models @
    output file = junk reset;
    library pgraph,optmum;
    graphset; optset;   

format 16,8;

rndseed 9137841;   @ resets random number generator so that identical sample is used 
                                    each time @


nt = 218;   @ nt is the number of observations in the original sample @
load qz1[nt,2] = ..\..\data2001\gdp.prn;   @ col. 1 = date; 
                                          col. 3 = growth of real gdp @
load qz2[nt,4] = ..\..\data2001\oildat2.prn;  @ col. 1 = date; col. 2 = growth; 
                                        col. 3 = oilmax; 
                                        col. 4 = oilmax over 3 years @
qz = qz1[.,1]~qz1[.,2]~qz2[.,2];
             @ col. 1 = date; col. 2 = gdp growth rate; col. 3 = percent change in oil price;   @


load qzquant[nt,2] = ..\..\data2001\quant.txt;
qzquant = qzquant~zeros(nt,8);
i = 1;
do until i > 8;
    qzquant[9:nt,i+2] = qzquant[9-i:nt-i,2];
i = i+1;
endo;

qzquant = qzquant[9:nt,3:cols(qzquant)];

   n = nt - 8;


y = qz[9:nt,2];   @ this is GDP growth rates, 49:II to 98:I @
x = zeros(n,1);     @ phantom entry to be removed shortly @
xp = 4;            @ xp is number of lags used for oil prices @
i = 1;
do until i > xp;
   x = x~qz[9-i:nt-i,3];  @ first xp cols of x will contain lagged oil prices @
i = i+1;
endo;


x = x[.,2:cols(x)];    @ removes phantom entry @

xlin = x;
yp = 4;     @ yp is number of lags used for gdp @
i = 1;
do until i > xp;
   xlin = xlin~qz[9-i:nt-i,2];   @ next yp cols of xlin contain lagged gdp @
i = i+1;
endo;



@ next read in the series for net oil price change @

xham = qz2[.,3];
xzham = xham[9-1:nt-1,1];
i = 2;
do until i > xp;
    xzham = xzham ~xham[9-i:nt-i,1];
i = i+1;
endo;


@ construct series for positive oil price change @
xmork = maxc(   ( qz[.,3]~zeros(nt,1) )'   );
xzmork = xmork[9-1:nt-1,1];
i = 2;
do until i > xp;
    xzmork = xzmork ~ xmork[9-i:nt-i,1];
i = i+1;
endo;

@ load Lee, Ni, and Ratti series @

load qzlee[nt,3] = ..\..\data2001\lee.dat;
xlee1 = zeros(n,1);
i = 1;
do until i > xp;
   xlee1 = xlee1~qzlee[9-i:nt-i,3];
i = i+1;
endo;
xlee1 = xlee1[.,2:cols(xlee1)];

    k = cols(x);                @ k is the number of nonlinear variables @
    klin = 1+cols(xlin);         @ klin is the total number of variables, nonlinear, linear
                                              and constant term @
    xwhole = ones(n,1)~xlin;   @ xwhole is (n x klin) @

/* ===========================================================
             CALCULATE AND REPORT SOME BASIC STATISTICS
=============================================================*/
xbar = meanc(x);
sigx = sqrt(meanc((x - xbar')^2));
"number of observations";;n;
"means of nonlinear explanatory variables";xbar';
"standard deviations";sigx';
xlinbar = meanc(xlin);
siglinx = sqrt(meanc((xlin - xlinbar')^2));
"means of all explanatory variables";xlinbar';
"standard deviations";siglinx';


kc=2;
proc(2) = lsq(yq,x1);
   @ This proc performs an OLS regression of yq on x1 @
local b,eps,sigols,sigmle,ss,ll,nq;
  b = invpd(x1'*x1)*x1'*yq;
   eps = yq - x1*b;
  sigols = sumc(eps^2)/(rows(x1)-cols(x1));
   sigmle = sumc(eps^2)/rows(x1);
   ss = sigols*invpd(x1'*x1);
   ll = -(n/2)*(1 + ln(2*pi)) - (n/2)*ln(sigmle);
 if kc > 1;
    ""; "ols coefficients";b';
     "ols see";;sqrt(sigols);
    "mle see";;sqrt(sigmle);
    "standard errors";sqrt(diag(ss))';
    "ols log likelihood";; ll;"";
endif;
retp(b,sigmle^.5);
endp;

 {bhat,sigmle} = lsq(y,xwhole);


xs = 0;ks=0;xswhole=0;
proc (0) = graph1dim(nofx,sigmult,xfix);
     @ this proc fills the (ks x nk) matrix xs and the (ks x klin) matrix xswhole
          with values needed to calculate the 
          effect of changing variable nofx with all others fixed at xfix.
          The variable nofx is varied from its value in xfix plus or minus 
           sigmult times its standard deviation @
      ks = 61;   @ ks is the number of function evaluations to be performed @
      xs = ones(ks, klin - 1) .* xfix';
      xs[.,nofx] = seqa(xfix[nofx,1] - sigmult*sigx[nofx,1], 
                                   (2*sigmult*sigx[nofx,1])/(ks-1), ks);
      xswhole = ones(ks,1) ~ xs;
      xs = xs[.,1:k];
endp;


/* ===========================================================
             PERFORM OLS REGRESSION AND TEST FOR NONLINEARITY
=============================================================*/

"";"Effects of simple linear model";
xgoose = xlinbar;
xgoose[4,1] = -5;
{btry,sigtry} = lsq(y,xwhole);
call graph1dim(3,1.5,xgoose);
yhatdn5 = xswhole*btry;
xgoose[4,1] = 0;
call graph1dim(3,1.5,xgoose);
yhat0 = xswhole*btry;
xgoose[4,1] = 5;
call graph1dim(3,1.5,xgoose);
yhatup5 = xswhole*btry;
"";"o(t-3)      o(t-4)=-5   o(t-4)=0    o(t-4)=+5";
xs[.,3]~yhatdn5~yhat0~yhatup5;
scale(-15 | 16, -1 | 2);
_ptek = "linear.tkf";
title("Linear");
let _pltype[3,1] = 1 6 3;
_pgrid = 3 | 0;
xy(xs[.,3],yhatdn5~yhat0~yhatup5);


"";"Effects of Mork's specification (price increases only)";
xzwhole = ones(n,1)~xzmork~xlin[.,xp+1:cols(xlin)];
xgoose = xlinbar;
xgoose[4,1] = 0;
{btry,sigtry} = lsq(y,xzwhole);
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( (xswhole[.,4]~zeros(ks,1))' );
yhatdn5 = xswhole*btry;
xgoose[4,1] = 0;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( (xswhole[.,4]~zeros(ks,1))' );
yhat0 = xswhole*btry;
xgoose[4,1] = 5;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( (xswhole[.,4]~zeros(ks,1))' );
yhatup5 = xswhole*btry;
"";"o(t-3)      o(t-4)=-5   o(t-4)=0    o(t-4)=+5";
xs[.,3]~yhatdn5~yhat0~yhatup5;
_ptek = "Mork.tkf";
title("Mork");
xy(xs[.,3],yhatdn5~yhat0~yhatup5);



"";"Effects of Hamilton's specification (net oil price)";
xzwhole = ones(n,1)~xzham~xlin[.,xp+1:cols(xlin)];
xgoose = xlinbar;
xgoose[4,1] = -5;
{btry,sigtry} = lsq(y,xzwhole);
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( ( (xswhole[.,4] + xswhole[.,5])~zeros(ks,1))' );
xswhole[.,5] = zeros(ks,1);
yhatdn5 = xswhole*btry;
xgoose[4,1] = 0;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] =maxc( ( (xswhole[.,4] + xswhole[.,5])~zeros(ks,1))' );
yhat0 = xswhole*btry;
xgoose[4,1] = 5;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( ( xswhole[.,4] ~zeros(ks,1))' );
yhatup5 = xswhole*btry;
"";"o(t-3)      o(t-4)=-5   o(t-4)=0    o(t-4)=+5";
xs[.,3]~yhatdn5~yhat0~yhatup5;
_ptek = "Hamilton.tkf";
title("Hamilton");
xy(xs[.,3],yhatdn5~yhat0~yhatup5);


"";"Effects of Lee, Ni, Ratti specification ";
xzwhole = ones(n,1)~xlee1~xlin[.,xp+1:cols(xlin)];
{btry,sigtry} = lsq(y,xzwhole);
xgoose = xlinbar;
xgoose[1,1] = 0;
xgoose[2,1] = 0;
xgoose[4,1] = 0;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( (xswhole[.,4]~zeros(ks,1))' );
xswhole[.,4] = xswhole[.,4] / 9.86;

yhatdn5 = xswhole*btry;
xgoose[4,1] = 0;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( (xswhole[.,4]~zeros(ks,1))' );
xswhole[.,4] = xswhole[.,4]/4.93;
yhat0 = xswhole*btry;
xgoose[4,1] = 5/10.46;
call graph1dim(3,1.5,xgoose);
xswhole[.,4] = maxc( (xswhole[.,4]~zeros(ks,1))' );
xswhole[.,4] = xswhole[.,4] / 7.87;
yhatup5 = xswhole*btry;
"";"o(t-3)      o(t-4)=-5   o(t-4)=0    o(t-4)=+5";
xs[.,3]~yhatdn5~yhat0~yhatup5;
_ptek = "lee.tkf";
title("Lee, Ni & Ratti");
xy(xs[.,3],yhatdn5~yhat0~yhatup5);








